/*
 * Author: Andrew Villadsen
 *
 * Copyright (c) 2017 Trustwave Holdings, Inc.
 * All Rights Reserved.
 *
 * This software is the confidential and proprietary information
 * of Trustwave Holdings, Inc.  Use of this software is governed by
 * the terms and conditions of the license statement and limited
 * warranty furnished with the software.
 *
 * IN PARTICULAR, YOU WILL INDEMNIFY AND HOLD TRUSTWAVE HOLDINGS INC.,
 * ITS RELATED COMPANIES AND ITS SUPPLIERS, HARMLESS FROM AND AGAINST
 * ANY CLAIMS OR LIABILITIES ARISING OUT OF OR RESULTING FROM THE USE,
 * MODIFICATION, OR DISTRIBUTION OF PROGRAMS OR FILES CREATED FROM,
 * BASED ON, AND/OR DERIVED FROM THIS SOURCE CODE FILE.
 */

import 'dart:async';

import 'package:angular_test/angular_test.dart';
import 'package:pageloader/objects.dart';

import 'package:tw_angular_test_utils/tw_angular_test_utils.dart';

class SavedSearchPO {
  @ByTagName('twa-menu-row')
  PageLoaderElement _menuButton;

  @ByTagName('twa-menu-popup')
  PageLoaderElement _menuSource;

  @Chain(const [const FirstByCss('twa-dialog'), const ByTagName('modal')])
  PageLoaderElement _saveDialog;

  @Chain(const [const ByTagName('twa-alert'), const ByTagName('modal')])
  PageLoaderElement _deleteAlert;

  Future<String> get label => _menuButton.visibleText;

  Future open() => _menuButton.click();

  Future<SavedSearchMenuPO> resolveMenu(NgTestFixture fixture) =>
      resolveOverlayPageObject(fixture, _menuSource, SavedSearchMenuPO);

  Future<SaveDialogPO> resolveSaveDialog(NgTestFixture fixture) =>
      resolveOverlayPageObject(fixture, _saveDialog, SaveDialogPO);

  Future<DeleteAlertPO> resolveDeleteAlert(NgTestFixture fixture) =>
      resolveOverlayPageObject(fixture, _deleteAlert, DeleteAlertPO);
}

class SavedSearchMenuPO {
  @Chain(const [
    const FirstByCss('twa-menu-group'),
    const ByTagName('twa-menu-row'),
    const WithVisibleText('Save As New Search')
  ])
  PageLoaderElement _saveAsNewSearch;

  @Chain(const [
    const FirstByCss('twa-menu-group'),
    const ByTagName('twa-menu-row'),
    const WithVisibleText('Save Changes')
  ])
  PageLoaderElement _saveChanges;

  @Chain(const [const FirstByCss('twa-menu-group'), const ByTagName('twa-menu-row'), const WithVisibleText('Delete')])
  PageLoaderElement _deleteSearch;

  @Chain(
      const [const FirstByCss('twa-menu-group'), const ByTagName('twa-menu-row'), const WithVisibleText('Syntax Tips')])
  PageLoaderElement _syntaxTips;

  Future<bool> get saveChangesDisabled => _saveChanges.classes.any((className) => className == 'disabled');

  Future<bool> get deleteDisabled => _deleteSearch.classes.any((className) => className == 'disabled');

  Future saveAsNew() => _saveAsNewSearch.click();

  Future saveChanges() => _saveChanges.click();

  Future delete() => _deleteSearch.click();

  Future syntaxTips() => _syntaxTips.click();

  @Chain(const [const ByCss('twa-menu-group:nth-child(2)'), const ByTagName('twa-menu-item')])
  List<SavedSearchItemPO> searches;
}

class SavedSearchItemPO {
  @ByTagName('twa-menu-row')
  PageLoaderElement _row;

  @ByClass('tw-check')
  @optional
  PageLoaderElement _check;

  Future<String> get name => _row.visibleText;

  Future select() => _row.click();

  bool get selected => _check != null;
}

class SaveDialogPO {
  @ByTagName('input')
  @optional
  PageLoaderElement _nameInput;

  @ByClass('input-readonly')
  @optional
  PageLoaderElement _nameReadonly;

  @Chain(const [const ByTagName('material-button'), const WithVisibleText('OK')])
  PageLoaderElement _saveButton;

  @Chain(const [const ByTagName('material-button'), const WithVisibleText('CANCEL')])
  PageLoaderElement _cancelButton;

  Future typeName(String name) => _nameInput.type(name);

  Future clearName() => _nameInput.clear();

  Future<String> get name => _nameInput?.properties['value'] ?? _nameReadonly?.visibleText;

  Future save() => _saveButton.click();

  Future cancel() => _cancelButton.click();
}

class DeleteAlertPO {
  @Chain(const [const ByTagName('material-button'), const WithVisibleText('Delete')])
  PageLoaderElement _deleteButton;

  @Chain(const [const ByTagName('material-button'), const WithVisibleText('CANCEL')])
  PageLoaderElement _cancelButton;

  Future delete() => _deleteButton.click();

  Future cancel() => _cancelButton.click();
}
